/*
    Copyright (C) 2000 Paul Davis 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: insert.h,v 1.30 2004/05/18 01:52:11 pauld Exp $
*/

#ifndef __ardour_insert_h__
#define __ardour_insert_h__

#include <vector>
#include <string>
#include <exception>

#include <sigc++/signal_system.h>
#include <ardour/ardour.h>
#include <ardour/redirect.h>
#include <ardour/plugin_state.h>

#include <glib.h>

class XMLNode;

namespace MIDI {
	class Port;
}

namespace ARDOUR {

class Session;
class Plugin;
class Route;

class Insert : public Redirect
{
  public:
	Insert(Session& s, Placement p);
	Insert(Session& s, string name, Placement p);
	Insert(Session& s, Placement p, int imin, int imax, int omin, int omax);
	
	virtual ~Insert() { }

	virtual void run (vector<Sample *>& bufs, jack_nframes_t nframes, jack_nframes_t offset) = 0;
	virtual void activate () {}
	virtual void deactivate () {}
};

class PortInsert : public Insert 
{
  public:
	PortInsert (Session&, Placement);
	PortInsert (Session&, const XMLNode&);
	PortInsert (const PortInsert&);
	~PortInsert ();

	XMLNode& get_state(void);
	int set_state(const XMLNode&);

	void init ();
	void run (vector<Sample *>& bufs, jack_nframes_t nframes, jack_nframes_t offset);

	jack_nframes_t latency();
};

struct PluginInsertState : public RedirectState
{
    PluginInsertState (std::string why) 
	    : RedirectState (why) {}
    ~PluginInsertState() {}

    PluginState plugin_state;
};

class PluginInsert : public Insert
{
  public:

	class wierd_plugin_configuration : public std::exception {
	  public:
		wierd_plugin_configuration(unsigned long s, unsigned long i, unsigned long o) 
			: streams (s), inputs (i), outputs (o) {}
			
		virtual const char *what() const throw() { return "wierd plugin configuration"; }

		unsigned long streams;
		unsigned long inputs;
		unsigned long outputs;
	};
	

	PluginInsert (Session&, Plugin&, Placement, Route&);
	PluginInsert (Session&, const XMLNode&);
	PluginInsert (const PluginInsert&);
	~PluginInsert ();

	static const string port_automation_node_name;
	
	XMLNode& get_state(void);
	int set_state(const XMLNode&);

	StateManager::State* state_factory (std::string why) const;
	Change restore_state (StateManager::State&);

	void run (vector<Sample *>& bufs, jack_nframes_t nframes, jack_nframes_t offset);
	void activate ();
	void deactivate ();

	unsigned long active_streams() const;

	void reset_midi_control (MIDI::Port*, bool);

	void set_parameter (unsigned long port, float val);

	AutoState get_port_automation_state (unsigned long port);
	void set_port_automation_state (unsigned long port, AutoState);
	
	float default_parameter_value (unsigned long which);

	Plugin& plugin(unsigned int num=0) const {
		if (num < _plugins.size()) { 
			return *_plugins[num];
		} else {
			return *_plugins[0]; // we always have one
		}
	}

	void set_count (unsigned int num);
	unsigned int get_count () { return _plugins.size(); }

	static unsigned long compute_count (Plugin&, Placement, ARDOUR::Route&, int index, unsigned long& streams);

	string describe_parameter (unsigned long);

	jack_nframes_t latency();

	void transport_stopped (jack_nframes_t now);
	void automation_snapshot (jack_nframes_t now);

  protected:
	void store_state (PluginInsertState&) const;

  private:

	void parameter_changed (unsigned long, float);
	
	vector<Plugin*> _plugins;
	void automation_run (vector<Sample *>& bufs, jack_nframes_t nframes, jack_nframes_t offset);
	void connect_and_run (vector<Sample *>& bufs, jack_nframes_t nframes, jack_nframes_t offset, bool with_auto, jack_nframes_t now = 0);

	void init ();
	void set_automatable ();
	void auto_state_changed (unsigned long which);
	void automation_list_creation_callback (unsigned long, AutomationList&);

	Plugin* plugin_factory (Plugin&);
};

}; /* namespace ARDOUR */

#endif /* __ardour_insert_h__ */
