/*
    Copyright (C) 2000 Paul Davis 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: ladspa_plugin.h,v 1.28 2004/07/28 17:43:59 pauld Exp $
*/

#ifndef __ardour_ladspa_plugin_h__
#define __ardour_ladspa_plugin_h__

#include <list>
#include <set>
#include <vector>
#include <string>
#include <dlfcn.h>

#include <midi++/controllable.h>
#include <sigc++/signal_system.h>

#include <jack/types.h>
#include <ardour/ladspa.h>
#include <ardour/stateful.h>
#include <ardour/plugin_state.h>
#include <ardour/plugin.h>
#include <ardour/ladspa_plugin.h>

using std::string;
using std::vector;
using std::list;

namespace ARDOUR {
class AudioEngine;
class Session;

class LadspaPlugin : public ARDOUR::Plugin
{
  public:
	LadspaPlugin (void *module, ARDOUR::AudioEngine&, ARDOUR::Session&, unsigned long index, jack_nframes_t sample_rate);
	LadspaPlugin (const LadspaPlugin &);
	~LadspaPlugin ();

	/* Plugin interface */
	
	unsigned long unique_id() const                  { return descriptor->UniqueID; }
	const char * label() const                       { return descriptor->Label; }
	const char * name() const                        { return descriptor->Name; }
	const char * maker() const                       { return descriptor->Maker; }
	unsigned long parameter_count() const            { return descriptor->PortCount; }
	float default_value (unsigned long port);
	jack_nframes_t latency() const;
	void set_parameter (unsigned long port, float val);
	float get_parameter (unsigned long port) const;
	int get_parameter_descriptor (unsigned long which, ParameterDescriptor&) const;
	std::set<unsigned long> automatable() const;
	unsigned long nth_parameter (unsigned long port, bool& ok) const;
	void activate () { 
		if (descriptor->activate) 
			descriptor->activate (handle);
	}
	void deactivate () {
		if (descriptor->deactivate) 
			descriptor->deactivate (handle);
	}
	void cleanup () {
		if (descriptor->cleanup) {
			descriptor->cleanup (handle);
		}
	}

	int connect_and_run (vector<Sample*>& bufs, size_t maxbuf, size_t& in, size_t& out, jack_nframes_t nframes, jack_nframes_t offset);
	void store_state (ARDOUR::PluginState&);
	void restore_state (ARDOUR::PluginState&);
	string describe_parameter (unsigned long);
	string state_node_name() const { return "ladspa"; }
	void print_parameter (unsigned long, char*, unsigned int len) const;

	bool parameter_is_audio(unsigned long) const;
	bool parameter_is_control(unsigned long) const;
	bool parameter_is_input(unsigned long) const;
	bool parameter_is_output(unsigned long) const;
	bool parameter_is_toggled(unsigned long) const;

	XMLNode& get_state();
	int set_state(const XMLNode& node);
	bool save_preset(string name);

	bool has_editor() const { return false; }

	/* LADSPA extras */

	LADSPA_Properties properties() const             { return descriptor->Properties; }
	unsigned long index() const                      { return _index; }
	const char * copyright() const                   { return descriptor->Copyright; }
	LADSPA_PortDescriptor port_descriptor(unsigned long i) const { return descriptor->PortDescriptors[i]; }
	const LADSPA_PortRangeHint * port_range_hints() const { return descriptor->PortRangeHints; }
	const char * const * port_names() const          { return descriptor->PortNames; }
	void set_gain (float gain) {
		descriptor->set_run_adding_gain (handle, gain);
	}
	void run_adding (unsigned long nsamples) {
		descriptor->run_adding (handle, nsamples);
	}
	void connect_port (unsigned long port, float *ptr) {
		descriptor->connect_port (handle, port, ptr);
	}

  private:
	void                    *module;
	const LADSPA_Descriptor *descriptor;
	LADSPA_Handle            handle;
	jack_nframes_t           sample_rate;
	LADSPA_Data             *control_data;
	LADSPA_Data             *shadow_data;
	LADSPA_Data             *latency_control_port;
	unsigned long           _index;
	
	void init (void *mod, unsigned long index, jack_nframes_t rate);
	void run (jack_nframes_t nsamples);
};
}

#endif /* __ardour_ladspa_plugin_h__ */
