/*
    Copyright (C) 2000 Paul Davis 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: plugin.h,v 1.7 2004/07/28 17:43:59 pauld Exp $
*/

#ifndef __ardour_ladspa_h__
#define __ardour_ladspa_h__

#include <midi++/controllable.h>
#include <sigc++/signal_system.h>

#include <jack/types.h>
#include <ardour/types.h>
#include <ardour/stateful.h>
#include <ardour/plugin_state.h>
#include <ardour/cycles.h>

#include <list>
#include <vector>
#include <set>
#include <map>

using std::string;
using std::vector;
using std::list;
using std::set;
using std::map;

namespace ARDOUR {

class AudioEngine;
class Session;

class PluginInfo {
  public:
	enum Type {
		LADSPA,
		VST
	};

	PluginInfo () { };
	PluginInfo (const PluginInfo &o)
		: name(o.name), n_inputs(o.n_inputs), n_outputs(o.n_outputs),
		path (o.path), index(o.index) {}
	~PluginInfo () { };
	string name;
	string category;
	int n_inputs;
	int n_outputs;
	Type type;

  private:
	friend class PluginManager;
	string path;
	unsigned long index;
};

class Plugin : public Stateful, public SigC::Object

{
  public:
	Plugin (ARDOUR::AudioEngine&, ARDOUR::Session&);
	Plugin (const Plugin&);
	~Plugin ();
	
	struct ParameterDescriptor {

	    /* essentially a union of LADSPA and VST info */

	    bool integer_step;
	    bool toggled;
	    bool logarithmic;
	    bool sr_dependent;
	    string label;
	    float lower;
	    float upper;
	    float step;
	    float smallstep;
	    float largestep;
	};

	virtual unsigned long unique_id() const = 0;
	virtual const char * label() const = 0;
	virtual const char * name() const = 0;
	virtual const char * maker() const = 0;
	virtual unsigned long parameter_count () const = 0;
	virtual float default_value (unsigned long port) = 0;
	virtual jack_nframes_t latency() const = 0;
	virtual void set_parameter (unsigned long which, float val) = 0;
	virtual float get_parameter(unsigned long which) const = 0;
	virtual int get_parameter_descriptor (unsigned long which, ParameterDescriptor&) const = 0;
	virtual unsigned long nth_parameter (unsigned long which, bool& ok) const = 0;
	virtual void activate () = 0;
	virtual void deactivate () = 0;
	virtual int connect_and_run (vector<Sample*>& bufs, size_t maxbuf, size_t& in, size_t& out, jack_nframes_t nframes, jack_nframes_t offset) = 0;
	virtual std::set<unsigned long> automatable() const = 0;
	virtual void store_state (ARDOUR::PluginState&) = 0;
	virtual void restore_state (ARDOUR::PluginState&) = 0;
	virtual string describe_parameter (unsigned long) = 0;
	virtual string state_node_name() const = 0;
	virtual void print_parameter (unsigned long, char*, unsigned int len) const = 0;

	virtual bool parameter_is_audio(unsigned long) const = 0;
	virtual bool parameter_is_control(unsigned long) const = 0;
	virtual bool parameter_is_input(unsigned long) const = 0;
	virtual bool parameter_is_output(unsigned long) const = 0;

	virtual bool save_preset(string name) = 0;
	virtual bool load_preset (const string preset_label);
	virtual list<string> get_presets();

	virtual bool has_editor() const = 0;

	SigC::Signal2<void,unsigned long,float> ParameterChanged;
	SigC::Signal1<void,Plugin *> GoingAway;

	void reset_midi_control (MIDI::Port*, bool);
	MIDI::Controllable *get_nth_midi_control (unsigned long);

	PluginInfo & get_info() { return _info; }
	void set_info (const PluginInfo &inf) { _info = inf; }

	ARDOUR::AudioEngine& engine() const { return _engine; }
	ARDOUR::Session& session() const { return _session; }

	void set_cycles (unsigned long c) { _cycles = c; }
	cycles_t cycles() const { return _cycles; }

  protected:
	ARDOUR::AudioEngine& _engine;
	ARDOUR::Session& _session;
	PluginInfo _info;
	unsigned long _cycles;
	map<string,string> 	 presets;
	bool save_preset(string name, string domain /* vst, ladspa etc. */);

	void setup_midi_controls ();

  private:
	struct MIDIPortControl : public MIDI::Controllable {
	    MIDIPortControl (Plugin&, unsigned long abs_port_id, MIDI::Port *,
			     float lower, float upper, bool toggled, bool logarithmic);

	    void set_value (float);

	    Plugin& plugin;
	    unsigned long absolute_port;
	    float upper;
	    float lower;
	    float range;
	    bool  toggled;
	    bool  logarithmic;
	};

  protected:
	vector<MIDIPortControl*> midi_controls;

	
};

/* this is actually defined in plugin_manager.cc */

Plugin * find_plugin(ARDOUR::Session&, string name, PluginInfo::Type);

} // namespace ARDOUR
 
#endif /* __ardour_plugin_h__ */
