/*
    Copyright (C) 2002 Paul Davis 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: slave.h,v 1.7 2004/06/29 17:48:10 pauld Exp $
*/

#ifndef __ardour_slave_h__
#define __ardour_slave_h__

#include <jack/jack.h>

#include <pthread.h>
#include <sigc++/signal_system.h>
#include <ardour/ardour.h>
#include <midi++/parser.h>

namespace MIDI {
	class Port;
}

namespace ARDOUR {
class Session;

class Slave {
  public:
	Slave() { _stopped = true; }
	virtual ~Slave() {}

	virtual jack_nframes_t current_audio_frame() = 0;
	virtual float current_speed () = 0;
	virtual bool locked() const = 0;
	virtual bool ok() const = 0;
	virtual bool starting() const { return false; }
	bool stopped () const { return _stopped; }

  protected:
	bool _stopped;
};


class MTC_Slave : public Slave, public SigC::Object {
  public:
	MTC_Slave (Session&, MIDI::Port&);
	~MTC_Slave ();

	jack_nframes_t current_audio_frame();
	float current_speed ();
	bool      locked() const;
	bool      ok() const;

	void          set_skip_limit (unsigned long);
	unsigned long get_skip_limit() const { return _mtc_skip_limit; }

  private:
	Session&    session;
	MIDI::Port& port;

	struct SafeTime {

	    int guard1;
	    SMPTE_Time  mtc;
	    jack_nframes_t   timestamp;
	    int guard2;

	    SafeTime() {
		    guard1 = 0;
		    guard2 = 0;
		    timestamp = 0;
	    }
	};

	SigC::Connection mtc_connection;
	SafeTime   current;
	jack_nframes_t  last_inbound_mtc_frame;   
	static const int mtc_accumulator_size = 16;
	float     mtc_speed_accumulator[mtc_accumulator_size];
	int       mtc_speed_counter;
	float     mtc_speed;
	unsigned long _mtc_skip_limit;
	unsigned long _mtc_skip_count;
	bool have_first_accumulated_mtc_speed;

	bool mtc_skip ();
	void update_mtc_time (const MIDI::byte *, bool);
	void update_mtc_status (MIDI::Parser::MTC_Status);
	void read_current (SafeTime *) const;
};

class ADAT_Slave : public Slave 
{
  public:
	ADAT_Slave () {}
	~ADAT_Slave () {}
	
	jack_nframes_t current_audio_frame() { return 0; }
	float current_speed () { return 0; }
	bool locked() const { return false; }
	bool ok() const { return false; }
};

class JACK_Slave : public Slave 
{
  public:
	JACK_Slave (jack_client_t*);
	~JACK_Slave ();
	
	jack_nframes_t current_audio_frame();
	float current_speed ();
	bool starting() const { return _starting; }
	bool locked() const;
	bool ok() const;

  private:
	jack_client_t* jack;
	float speed;
	bool _starting;
};

} /* namespace */

#endif /* __ardour_slave_h__ */
